/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package FunctionLayer

import DataLayer.DataMapper
import FunctionLayer.StanfordParser.SentimentAnalyzerTest
import FunctionLayer.StanfordParser.SentimentValueCache
import com.google.common.base.Stopwatch
import com.google.common.collect.MapMaker
import edu.stanford.nlp.ie.AbstractSequenceClassifier
import edu.stanford.nlp.ie.crf.CRFClassifier
import edu.stanford.nlp.ling.CoreLabel
import edu.stanford.nlp.parser.lexparser.LexicalizedParser
import edu.stanford.nlp.pipeline.Annotation
import edu.stanford.nlp.pipeline.CoreDocument
import edu.stanford.nlp.pipeline.StanfordCoreNLP
import edu.stanford.nlp.tagger.maxent.MaxentTagger
import edu.stanford.nlp.trees.GrammaticalStructureFactory
import edu.stanford.nlp.trees.TreebankLanguagePack
import kotlinx.coroutines.*
import java.io.IOException
import java.io.UnsupportedEncodingException
import java.lang.Runnable
import java.net.DatagramPacket
import java.net.DatagramSocket
import java.net.InetAddress
import java.net.SocketException
import java.sql.SQLException
import java.util.*
import java.util.concurrent.*
import java.util.function.Consumer
import java.util.logging.Level
import java.util.logging.Logger
import kotlin.collections.ArrayList

/**
 *
 * @author install1
 */
class Datahandler {
    private val lHMSMX: LinkedHashMap<Any?, Any?> = LinkedHashMap<Any?, Any?>()
    private val stopwatch: Stopwatch
    fun shiftReduceParserInitiate() {
        val cdl = CountDownLatch(2)
        Thread(Runnable {
            try {
                classifier = CRFClassifier.getClassifierNoExceptions(nerModel)
            } catch (ex: ClassCastException) {
                Logger.getLogger(Datahandler::class.java.name).log(Level.SEVERE, null, ex)
            }
            cdl.countDown()
        }).start()
        Thread(Runnable {
            propsSentiment.setProperty("parse.model", lexParserEnglishRNN)
            propsSentiment.setProperty("sentiment.model", sentimentModel)
            propsSentiment.setProperty("parse.maxlen", "90")
            propsSentiment.setProperty("threads", "5")
            propsSentiment.setProperty("pos.maxlen", "90")
            propsSentiment.setProperty("tokenize.maxlen", "90")
            propsSentiment.setProperty("ssplit.maxlen", "90")
            propsSentiment.setProperty("annotators", "tokenize,ssplit,pos,parse,sentiment,lemma,stopword") //coref too expensive memorywise
            propsSentiment.setProperty("customAnnotatorClass.stopword", "FunctionLayer.StopwordAnnotator")
            propsSentiment.setProperty(StopwordAnnotator.STOPWORDS_LIST, customStopWordList)
            propsSentiment.setProperty("tokenize.options", "untokenizable=firstKeep")
            pipelineSentiment = StanfordCoreNLP(propsSentiment)
            tagger = MaxentTagger(taggerPath)
            cdl.countDown()
        }).start()
        lp = LexicalizedParser.loadModel(lexParserEnglishRNN, *options)

        tlp = lp.getOp().langpack()
        gsf = tlp.grammaticalStructureFactory()
        try {
            cdl.await()
        } catch (ex: InterruptedException) {
            //System.out.println("cdl await interrupted: " + ex.getLocalizedMessage() + "\n");
        }
        println("finished shiftReduceParserInitiate\n")
    }

    fun updateStringCache() {
        try {
            checkIfUpdateStrings()
        } catch (ex: CustomError) {
            Logger.getLogger(Datahandler::class.java.name).log(Level.SEVERE, null, ex)
        }
    }

    @get:Throws(SQLException::class, IOException::class, CustomError::class)
    private val cache: Map<Int, String?>
        private get() = DataMapper.getAllStrings()

    @Throws(SQLException::class, IOException::class)
    fun initiateMYSQL() {
        try {
            DataMapper.createTables()
            stringCache.putAll(cache)
            // lHMSMX = DataMapper.getAllRelationScores();
        } catch (ex: CustomError) {
            Logger.getLogger(Datahandler::class.java
                    .name).log(Level.SEVERE, null, ex)
        }
    }

    fun instantiateAnnotationMapJMWE() {
        if (!stringCache.isEmpty()) {
            val jmweAnnotation = PipelineJMWESingleton.INSTANCE.getJMWEAnnotation(stringCache.values)
            for ((key, value) in jmweAnnotation) {
                jmweAnnotationCache[key] = value
            }
        }
    }

    fun instantiateAnnotationMap() {
        if (!stringCache.isEmpty()) {
            val Annotationspipeline = MapMaker().concurrencyLevel(4).makeMap<String?, Annotation>()
            val AnnotationspipelineSentiment = MapMaker().concurrencyLevel(4).makeMap<String?, Annotation>()
            runBlocking {
                val job = launch(Dispatchers.Default) {
                    for (str in stringCache.values) {
                        val strAnno = Annotation(str)
                        strAnno.compact()
                        Annotationspipeline[str] = strAnno
                        val strAnno2 = Annotation(str)
                        strAnno2.compact()
                        AnnotationspipelineSentiment[str] = strAnno2
                    }
                    yield()
                }
                job.join();
            }
            val coreDocumentpipelineMap = getMultipleCoreDocumentsWaySuggestion(stringCache.values, pipeline)
            pipeline.annotate(Annotationspipeline.values)
            pipelineSentiment!!.annotate(AnnotationspipelineSentiment.values)
            runBlocking {
                val job = launch(Dispatchers.Default) {
                    for (i in Annotationspipeline.entries) {
                        i.value.compact()
                        pipelineAnnotationCache[i.key] = i.value
                    }
                    yield()
                }
                job.join()
            }
            runBlocking {
                val job = launch(Dispatchers.Default) {
                    for (i in AnnotationspipelineSentiment.entries) {
                        i.value.compact()
                        pipelineSentimentAnnotationCache[i.key] = i.value
                    }
                    yield()
                }
                job.join()
            }
            runBlocking {
                val job = launch(Dispatchers.Default) {
                    for (i in coreDocumentpipelineMap.entries) {
                        coreDocumentAnnotationCache[i.key] = i.value
                    }
                    yield()
                }
                job.join()
            }
        }
    }

    private fun futuresReturnOverallEvaluation(similarityMatrixes: List<SimilarityMatrix?>): ConcurrentMap<Int?, String?> {
        var strmapreturn = MapMaker().concurrencyLevel(6).makeMap<Int?, String?>()
        if (!similarityMatrixes.isEmpty()) {
            var iterator = 0
            for (SMX in similarityMatrixes) {
                val scoreRelationNewMsgToRecentMsg = SMX!!.distance
                if (scoreRelationNewMsgToRecentMsg > 0.0) {
                    strmapreturn = addSMXToMapReturn(strmapreturn, SMX)
                }
                //System.out.println("scoreRelationNewMsgToRecentMsg: " + scoreRelationNewMsgToRecentMsg + "\niterator: " + iterator);
                iterator++
            }
        }
        return strmapreturn
    }

    private fun addSMXToMapReturn(strmapreturn: ConcurrentMap<Int?, String?>, SMX: SimilarityMatrix?): ConcurrentMap<Int?, String?> {
        if (!strmapreturn.containsValue(SMX!!.primaryString)) {
            strmapreturn[strmapreturn.size] = SMX.primaryString
            val transmittedStr = SMX.secondaryString
            val cacheValue1 = SMX.cacheValue1
            val cacheValue2 = SMX.cacheValue2
            if (cacheValue1 != null && !sentimentCachingMap.keys.contains(SMX.primaryString)) {
                sentimentCachingMap[SMX.secondaryString] = SMX.cacheValue1
            }
            if (cacheValue2 != null && !sentimentCachingMap.keys.contains(transmittedStr)) {
                sentimentCachingMap[transmittedStr] = SMX.cacheValue2
            }
        }
        return strmapreturn
    }


    private fun StrComparringNoSentenceRelationMap(strCacheLocal: ConcurrentMap<Int, String?>, strCollection: Collection<String?>, localJMWEMap: ConcurrentMap<String, Annotation>,
                                                   localPipelineAnnotation: ConcurrentMap<String?, Annotation>, localPipelineSentimentAnnotation: ConcurrentMap<String?, Annotation>,
                                                   localCoreDocumentMap: ConcurrentMap<String, CoreDocument>): List<SimilarityMatrix?> {

        //TODO here
        val distance_requirement = 15500.0
        val prefix_size = 150
        val smxReturnList: ArrayList<SimilarityMatrix> = ArrayList<SimilarityMatrix>()
        runBlocking {
            val job = launch(Dispatchers.Default) {
                for (j in strCollection) {
                    for (i in strCollection) {
                        if (j != i) {
                            val SMXInit = SimilarityMatrix(j, i)
                            val sentimentCacheStr1 = sentimentCachingMap.getOrDefault(i, null)
                            val sentimentCacheStr = sentimentCachingMap.getOrDefault(j, null)
                            var sentimentAnalyzerTest: SentimentAnalyzerTest? = null
                            if (stringCache.size < prefix_size) {
                                sentimentAnalyzerTest = SentimentAnalyzerTest(j, i, SMXInit,
                                        localJMWEMap[j], localJMWEMap[i], localPipelineAnnotation[j],
                                        localPipelineAnnotation[i], localPipelineSentimentAnnotation[j],
                                        localPipelineSentimentAnnotation[i], localCoreDocumentMap[j], localCoreDocumentMap[i],
                                        sentimentCacheStr, sentimentCacheStr1)
                            } else {
                                sentimentAnalyzerTest = SentimentAnalyzerTest(j, i, SMXInit,
                                        localJMWEMap[j], jmweAnnotationCache[i], localPipelineAnnotation[j],
                                        pipelineAnnotationCache[i], localPipelineSentimentAnnotation[j],
                                        pipelineSentimentAnnotationCache[i], localCoreDocumentMap[j],
                                        coreDocumentAnnotationCache[i], sentimentCacheStr, sentimentCacheStr1)
                            }
                            val call = sentimentAnalyzerTest.call();
                            if (call != null && call.distance > distance_requirement) {
                                smxReturnList.add(call)
                            }
                        }
                    }
                }
                yield()
            }
            job.join()
        }

        return smxReturnList
    }

    private fun stringIteratorComparator(strmap: ConcurrentMap<Int?, String?>,
                                         strCacheLocal: ConcurrentMap<Int, String?>, localJMWEMap: ConcurrentMap<String, Annotation>,
                                         localPipelineAnnotation: ConcurrentMap<String?, Annotation>, localPipelineSentimentAnnotation: ConcurrentMap<String?, Annotation>,
                                         localCoreDocumentMap: ConcurrentMap<String, CoreDocument>): ConcurrentMap<Int?, String?> {
        //System.out.println("strmap siuze: " + strmap.size());
        val ComparringNoSentenceRelationMap: List<SimilarityMatrix> = StrComparringNoSentenceRelationMap(strCacheLocal, strmap.values,
                localJMWEMap, localPipelineAnnotation, localPipelineSentimentAnnotation, localCoreDocumentMap) as List<SimilarityMatrix>
        Collections.sort(ComparringNoSentenceRelationMap, Comparator<SimilarityMatrix> { e1: SimilarityMatrix, e2: SimilarityMatrix -> e1.primaryString.compareTo(e2.primaryString) })
        //System.out.println("strmapreturn size: " + strmapreturn.size());
        return futuresReturnOverallEvaluation(ComparringNoSentenceRelationMap)
    }

    private fun removeNonSensicalStrings(strmap: ConcurrentMap<Int?, String?>): ConcurrentMap<Int?, String?> {
        val strCacheLocal = stringCache
        val localJMWEMap = getMultipleJMWEAnnotation(strmap.values)
        val localPipelineAnnotation = getMultiplePipelineAnnotation(strmap.values)
        val localPipelineSentimentAnnotation = getMultiplePipelineSentimentAnnotation(strmap.values)
        val localCoreDocumentMap = getMultipleCoreDocumentsWaySuggestion(strmap.values, pipeline)
        return stringIteratorComparator(strmap, strCacheLocal, localJMWEMap, localPipelineAnnotation, localPipelineSentimentAnnotation, localCoreDocumentMap)
    }

    @Synchronized
    @Throws(CustomError::class)
    fun checkIfUpdateStrings() {
        if (stopwatch.elapsed(TimeUnit.SECONDS) >= EXPIRE_TIME_IN_SECONDS || !stopwatch.isRunning) {
            var str = MessageResponseHandler.getStr()
            println("str size: " + str.size)
            str = filterContent(str)
            str = removeNonSensicalStrings(str)
            //System.out.println("removeNonSensicalStrings str size POST: " + str.size() + "\n");
            str = annotationCacheUpdate(str)
            println("""
    annotationCacheUpdate str size POST: ${str.size}
    
    """.trimIndent())
            val strf = str
            if (!stringCache.isEmpty()) {
                Thread(Runnable {
                    try {
                        DataMapper.InsertMYSQLStrings(strf)
                    } catch (ex: CustomError) {
                        Logger.getLogger(Datahandler::class.java
                                .name).log(Level.SEVERE, null, ex)
                    }
                    MessageResponseHandler.setStr(MapMaker().concurrencyLevel(6).makeMap())
                }).start()
            } else {
                try {
                    DataMapper.InsertMYSQLStrings(strf)
                } catch (ex: CustomError) {
                    Logger.getLogger(Datahandler::class.java
                            .name).log(Level.SEVERE, null, ex)
                }
                MessageResponseHandler.setStr(MapMaker().concurrencyLevel(6).makeMap())
            }
            if (!stopwatch.isRunning) {
                stopwatch.start()
            } else {
                stopwatch.reset()
            }
        }
    }

    private fun trimString(str: String): String {
        var str = str
        str = str.trim { it <= ' ' }
        if (str.startsWith("<@")) {
            str = str.substring(str.indexOf("> ") + 2)
        }
        return str
    }

    private fun getResponseFutures(strF: String): String {
        val values_copy: List<String?> = ArrayList(stringCache.values)
        Collections.shuffle(values_copy)
        var preRelationUserCounters = -155000.0
        val concurrentRelations: MutableList<String?> = arrayListOf()
        runBlocking {
            val job = launch(Dispatchers.Default) {
                for (str1 in values_copy) {
                    if (strF != str1) {
                        val sentimentCacheStr1 = sentimentCachingMap.getOrDefault(str1, null)
                        val worker: Callable<SimilarityMatrix> = SentimentAnalyzerTest(strF, str1, SimilarityMatrix(strF, str1),
                                strAnnoJMWE, jmweAnnotationCache[str1], strAnno,
                                pipelineAnnotationCache[str1], strAnnoSentiment,
                                pipelineSentimentAnnotationCache[str1], coreDoc, coreDocumentAnnotationCache[str1],
                                null, sentimentCacheStr1)
                        try {
                            val getSMX = worker.call()
                            if (getSMX != null) {
                                val scoreRelationLastUserMsg = getSMX.distance
                                if (scoreRelationLastUserMsg > preRelationUserCounters) {
                                    preRelationUserCounters = scoreRelationLastUserMsg
                                    concurrentRelations.add(getSMX.secondaryString)
                                }
                            }
                        } catch (ex: Exception) {
                            Logger.getLogger(Datahandler::class.java.name).log(Level.SEVERE, null, ex)
                        }
                    }
                }
                yield()
            }
            job.join()
        }

        val SB = StringBuilder()
        val randomLenghtPermit = strF.length * (Math.random() * Math.random() * Math.random() * 5)
        Collections.reverse(concurrentRelations)
        if (concurrentRelations.isEmpty()) {
            return "failure, preventing stuckness"
        }

        val firstRelation = concurrentRelations[0]
        runBlocking {
            val job = launch(Dispatchers.Default) {
                for (secondaryRelation in concurrentRelations) {
                    if (SB.toString().length > randomLenghtPermit && !SB.toString().isEmpty()) {
                        break
                    }
                    val append = appendToString(firstRelation, secondaryRelation)
                    if (append) {
                        SB.append(secondaryRelation).append(" ")
                    }
                }
                yield()
            }
            job.join()
        }
        return SB.toString()
    }

    private fun appendToString(firstRelation: String?, secondaryRelation: String?): Boolean {
        if (firstRelation == secondaryRelation) {
            return true
        }
        val scoreRelationStrF = getScoreRelationStrF(firstRelation, secondaryRelation)
        return if (scoreRelationStrF > 1900) {
            true
        } else false
    }

    @Throws(CustomError::class)
    fun getResponseMsg(str: String): String {
        val strF = trimString(str)
        getSingularAnnotation(strF)
        return getResponseFutures(strF)
    }

    fun getSingularAnnotation(str: String?) {
        strAnno = Annotation(str)
        strAnno!!.compact()
        pipeline.annotate(strAnno)
        strAnnoSentiment = Annotation(str)
        strAnnoSentiment!!.compact()
        pipelineSentiment!!.annotate(strAnnoSentiment)
        val notactualList: MutableList<String?> = arrayListOf()
        notactualList.add(str)
        val jmweAnnotation = PipelineJMWESingleton.INSTANCE.getJMWEAnnotation(notactualList)
        strAnnoJMWE = jmweAnnotation.values.iterator().next()
        strAnnoJMWE.compact()
        val coreDocument = CoreDocument(str)
        pipeline.annotate(coreDocument)
        coreDoc = coreDocument
    }

    private fun getScoreRelationStrF(str: String?, mostRecentMsg: String?): Double {
        val SMX = SimilarityMatrix(str, mostRecentMsg)
        val cacheSentiment1 = sentimentCachingMap.getOrDefault(str, null)
        val cacheSentiment2 = sentimentCachingMap.getOrDefault(mostRecentMsg, null)
        val worker: Callable<SimilarityMatrix> = SentimentAnalyzerTest(str, mostRecentMsg, SMX,
                strAnnoJMWE, jmweAnnotationCache[mostRecentMsg], strAnno,
                pipelineAnnotationCache[mostRecentMsg], strAnnoSentiment,
                pipelineSentimentAnnotationCache[mostRecentMsg], coreDoc, coreDocumentAnnotationCache[mostRecentMsg], cacheSentiment1, cacheSentiment2)
        var callSMX: SimilarityMatrix? = null
        try {
            callSMX = worker.call()
        } catch (ex: Exception) {
            Logger.getLogger(Datahandler::class.java
                    .name).log(Level.SEVERE, null, ex)
        }
        return callSMX?.distance ?: 0.0
    }

    private fun annotationCacheUpdate(strmap: ConcurrentMap<Int?, String?>): ConcurrentMap<Int?, String?> {
        val jmweAnnotation = PipelineJMWESingleton.INSTANCE.getJMWEAnnotation(strmap.values)
        for ((key, value) in jmweAnnotation) {
            jmweAnnotationCache[key] = value
        }
        val Annotationspipeline = MapMaker().concurrencyLevel(6).makeMap<String?, Annotation>()
        val AnnotationspipelineSentiment = MapMaker().concurrencyLevel(6).makeMap<String?, Annotation>()
        val coreDocumentpipelineMap = getMultipleCoreDocumentsWaySuggestion(strmap.values, pipeline)
        runBlocking {
            val job = launch(Dispatchers.Default) {
                for (str in strmap.values) {
                    val strAnno1 = Annotation(str)
                    Annotationspipeline[str] = strAnno1
                    val strAnno2 = Annotation(str)
                    AnnotationspipelineSentiment[str] = strAnno2
                    stringCache[stringCache.size + 1] = str
                }
                yield()
            }
            job.join()
        }
        pipeline.annotate(Annotationspipeline.values)
        pipelineSentiment!!.annotate(AnnotationspipelineSentiment.values)
        runBlocking {
            val job = launch(Dispatchers.Default) {
                for (pipelineEntry in Annotationspipeline.entries) {
                    if (pipelineEntry != null) {
                        pipelineAnnotationCache[pipelineEntry.key] = pipelineEntry.value
                    }
                }
                yield()
            }
            job.join()
        }
        runBlocking {
            val job = launch(Dispatchers.Default) {
                for (pipelineEntry in AnnotationspipelineSentiment.entries) {
                    if (pipelineEntry != null) {
                        pipelineSentimentAnnotationCache[pipelineEntry.key] = pipelineEntry.value
                    }
                }
                yield()
            }
            job.join()
        }
        runBlocking {
            val job = launch(Dispatchers.Default) {
                for (coreDocumentEntry in coreDocumentpipelineMap.entries) {
                    coreDocumentAnnotationCache[coreDocumentEntry.key] = coreDocumentEntry.value
                }
                yield()
            }
            job.join()
        }
        return strmap
    }

    val messageOverHead: Int
        get() = stringCache.values.size - stringCache.values.size / 10

    fun update_autismo_socket_msg() {
        try {
            try {
                DatagramSocket(48480).use { serverSocket ->
                    DatagramSocket(48471).use { serverSocket1 ->
                        val receiveData = ByteArray(4096)
                        val IPAddress = InetAddress.getByName("135.125.188.157") //later moving autism bot 1 and 3 to OVH from SYS
                        var receivePacket = DatagramPacket(receiveData, receiveData.size)
                        while (true) {
                            serverSocket.receive(receivePacket)
                            var sentence = String(receivePacket.data, 0, receivePacket.length)
                            sentence = sentence.replace("clientmessage:", "")
                            var getResponseMsg = getResponseMsg(sentence)
                            var sendData = getResponseMsg.toByteArray(charset("UTF-8"))
                            var sendPacket = DatagramPacket(sendData, sendData.size, IPAddress, 48479)
                            serverSocket.send(sendPacket)
                            receivePacket = DatagramPacket(receiveData, receiveData.size)
                            serverSocket1.receive(receivePacket)
                            sentence = String(receivePacket.data, 0, receivePacket.length)
                            sentence = sentence.replace("clientmessage:", "")
                            getResponseMsg = getResponseMsg(sentence)
                            sendData = getResponseMsg.toByteArray(charset("UTF-8"))
                            sendPacket = DatagramPacket(sendData, sendData.size, IPAddress, 48476)
                            serverSocket1.send(sendPacket)
                        }
                    }
                }
            } catch (ex: CustomError) {
                Logger.getLogger(Datahandler::class.java.name).log(Level.SEVERE, null, ex)
            }
        } catch (ex: SocketException) {
            Logger.getLogger(Datahandler::class.java.name).log(Level.SEVERE, null, ex)
        } catch (ex: UnsupportedEncodingException) {
            Logger.getLogger(Datahandler::class.java.name).log(Level.SEVERE, null, ex)
        } catch (ex: IOException) {
            Logger.getLogger(Datahandler::class.java.name).log(Level.SEVERE, null, ex)
        }
    }

    private class AnnotationCollector<T> : Consumer<T> {
        val annotationsT: MutableList<T?> = arrayListOf()
        override fun accept(ann: T) {
            //System.out.println("adding ann: " + ann.toString());
            annotationsT.add(ann)
        }

        companion object {
            var i = 0
        }
    }

    companion object {
        val EXPIRE_TIME_IN_SECONDS = TimeUnit.SECONDS.convert(10, TimeUnit.MINUTES)
        val EXPIRE_TIME_IN_SECONDS1 = TimeUnit.SECONDS.convert(10, TimeUnit.HOURS)

        @JvmField
        var instance = Datahandler()
        private var strAnno: Annotation? = null
        private var strAnnoSentiment: Annotation? = null
        private lateinit var strAnnoJMWE: Annotation
        private var coreDoc: CoreDocument? = null
        private val stringCache = MapMaker().concurrencyLevel(6).makeMap<Int, String?>()
        private lateinit var pipelineAnnotationCache: ConcurrentMap<String?, Annotation>
        private lateinit var pipelineSentimentAnnotationCache: ConcurrentMap<String?, Annotation>
        private lateinit var jmweAnnotationCache: ConcurrentMap<String, Annotation>
        private lateinit var coreDocumentAnnotationCache: ConcurrentMap<String, CoreDocument>
        private val sentimentCachingMap = MapMaker().concurrencyLevel(6).makeMap<String, SentimentValueCache>()
        private const val similar = ""
        private const val shiftReduceParserPath = "edu/stanford/nlp/models/srparser/englishSR.ser.gz"
        private const val sentimentModel = "edu/stanford/nlp/models/sentiment/sentiment.ser.gz"
        private const val lexParserEnglishRNN = "edu/stanford/nlp/models/lexparser/englishRNN.ser.gz"
        private const val taggerPath = "edu/stanford/nlp/models/pos-tagger/english-left3words/english-left3words-distsim.tagger"
        private const val nerModel = "edu/stanford/nlp/models/ner/english.all.3class.caseless.distsim.crf.ser.gz"
        private const val nerModel2 = "edu/stanford/nlp/models/ner/english.conll.4class.caseless.distsim.crf.ser.gz"
        private const val nerModel3 = "edu/stanford/nlp/models/ner/english.muc.7class.caseless.distsim.crf.ser.gz"
        private const val customStopWordList = "start,starts,period,periods,a,an,and,are,as,at,be,but,by,for,if,in,into,is,it,no,not,of,on,or,such,that,the,their,then,there,these,they,this,to,was,will,with"

        @JvmStatic
        var tagger: MaxentTagger? = null
            private set
        private val options = arrayOf("-maxLength", "100")
        private val props = Properties()
        private val propsSentiment = Properties()

        @JvmStatic
        var gsf: GrammaticalStructureFactory? = null
            private set
        private lateinit var lp: LexicalizedParser
        private lateinit var tlp: TreebankLanguagePack
        private lateinit var classifier: AbstractSequenceClassifier<CoreLabel>

        public fun getPipeLine(): StanfordCoreNLP {
            return pipeline
        }

        // set up Stanford CoreNLP pipeline
        @JvmStatic
        val pipeline = pipeLineSetUp
        private var pipelineSentiment: StanfordCoreNLP? = null

        private val pipeLineSetUp: StanfordCoreNLP
            private get() {
                props.setProperty("annotators", "tokenize,ssplit,pos,lemma,ner,parse")
                props.setProperty("parse.model", shiftReduceParserPath)
                props.setProperty("parse.maxlen", "90")
                props.setProperty("parse.binaryTrees", "true")
                props.setProperty("threads", "5")
                props.setProperty("pos.maxlen", "90")
                props.setProperty("tokenize.maxlen", "90")
                props.setProperty("ssplit.maxlen", "90")
                props.setProperty("lemma.maxlen", "90")
                props.setProperty("ner.model", "$nerModel,$nerModel2,$nerModel3")
                props.setProperty("ner.combinationMode", "HIGH_RECALL")
                props.setProperty("regexner.ignorecase", "true")
                props.setProperty("ner.fine.regexner.ignorecase", "true")
                props.setProperty("tokenize.options", "untokenizable=firstDelete")
                return StanfordCoreNLP(props)
            }

        @JvmStatic
        fun getClassifier(): AbstractSequenceClassifier<CoreLabel>? {
            return classifier
        }

        fun setClassifier(classifier: AbstractSequenceClassifier<CoreLabel>?) {
            if (classifier != null) {
                Companion.classifier = classifier
            }
        }

        private fun getMultipleJMWEAnnotation(str: Collection<String?>): ConcurrentMap<String, Annotation> {
            return PipelineJMWESingleton.INSTANCE.getJMWEAnnotation(str)
        }

        private fun getMultiplePipelineAnnotation(str: Collection<String?>): ConcurrentMap<String?, Annotation> {
            val pipelineAnnotationMap = MapMaker().concurrencyLevel(2).makeMap<String?, Annotation>()
            for (str1 in str) {
                val strAnno1 = Annotation(str1)
                pipelineAnnotationMap[str1] = strAnno1
            }
            pipeline.annotate(pipelineAnnotationMap.values)
            return pipelineAnnotationMap
        }

        private fun getMultiplePipelineSentimentAnnotation(str: Collection<String?>): ConcurrentMap<String?, Annotation> {
            val pipelineAnnotationMap = MapMaker().concurrencyLevel(2).makeMap<String?, Annotation>()
            for (str1 in str) {
                val strAnno1 = Annotation(str1)
                pipelineAnnotationMap[str1] = strAnno1
            }
            pipelineSentiment!!.annotate(pipelineAnnotationMap.values)
            return pipelineAnnotationMap
        }

        fun filterContent(str: ConcurrentMap<Int?, String?>): ConcurrentMap<Int?, String?> {
            val strlistreturn = MapMaker().concurrencyLevel(2).makeMap<Int?, String?>()
            str.values.forEach(Consumer {
                var str1: String = ""
                if (!str1.isEmpty() && str1.length > 3) {
                    str1 = str1.trim { it <= ' ' }
                    if (str1.contains("PM*")) {
                        str1 = str1.substring(str1.indexOf("PM*") + 3)
                    }
                    if (str1.contains("AM*")) {
                        str1 = str1.substring(str1.indexOf("AM*") + 3)
                    }
                    /*
                if (str1.contains("?") || str1.contains("°"))
                {
                    if (!str1.contains("http"))
                    {
                        str1 = str1.replace("?", " <:wlenny:514861023002624001> ");
                        str1 = str1.replace("°", " <:wlenny:514861023002624001> ");
                    }
                }
                */if (str1.contains("(Counter-Terrorist)")) {
                        str1 = str1.replace("(Counter-Terrorist)", " ")
                    }
                    if (str1.contains("(Terrorist)")) {
                        str1 = str1.replace("(Terrorist)", " ")
                    }
                    if (str1.contains("(Spectator)")) {
                        str1 = str1.replace("(Spectator)", " ")
                    }
                    if (str1.contains("*DEAD*")) {
                        str1 = str1.replace("*DEAD*", " ")
                    }
                    if (str1.contains("{red}")) {
                        str1 = str1.replace("{red}", " ")
                    }
                    if (str1.contains("{orange}")) {
                        str1 = str1.replace("{orange}", " ")
                    }
                    if (str1.contains("{yellow}")) {
                        str1 = str1.replace("{yellow}", " ")
                    }
                    if (str1.contains("{green}")) {
                        str1 = str1.replace("{green}", " ")
                    }
                    if (str1.contains("{lightblue}")) {
                        str1 = str1.replace("{lightblue}", " ")
                    }
                    if (str1.contains("{blue}")) {
                        str1 = str1.replace("{blue}", " ")
                    }
                    if (str1.contains("{purple}")) {
                        str1 = str1.replace("{purple}", " ")
                    }
                    if (str1.contains("{white}")) {
                        str1 = str1.replace("{white}", " ")
                    }
                    if (str1.contains("{fullblue}")) {
                        str1 = str1.replace("{fullblue}", " ")
                    }
                    if (str1.contains("{cyan}")) {
                        str1 = str1.replace("{cyan}", " ")
                    }
                    if (str1.contains("{lime}")) {
                        str1 = str1.replace("{lime}", " ")
                    }
                    if (str1.contains("{deeppink}")) {
                        str1 = str1.replace("{deeppink}", " ")
                    }
                    if (str1.contains("{slategray}")) {
                        str1 = str1.replace("{slategray}", " ")
                    }
                    if (str1.contains("{dodgerblue}")) {
                        str1 = str1.replace("{dodgerblue}", " ")
                    }
                    if (str1.contains("{black}")) {
                        str1 = str1.replace("{black}", " ")
                    }
                    if (str1.contains("{orangered}")) {
                        str1 = str1.replace("{orangered}", " ")
                    }
                    if (str1.contains("{darkorchid}")) {
                        str1 = str1.replace("{darkorchid}", " ")
                    }
                    if (str1.contains("{pink}")) {
                        str1 = str1.replace("{pink}", " ")
                    }
                    if (str1.contains("{lightyellow}")) {
                        str1 = str1.replace("{lightyellow}", " ")
                    }
                    if (str1.contains("{chocolate}")) {
                        str1 = str1.replace("{chocolate}", " ")
                    }
                    if (str1.contains("{beige}")) {
                        str1 = str1.replace("{beige}", " ")
                    }
                    if (str1.contains("{azure}")) {
                        str1 = str1.replace("{azure}", " ")
                    }
                    if (str1.contains("{yellowgreen}")) {
                        str1 = str1.replace("{yellowgreen}", " ")
                    }
                    str1 = str1.trim { it <= ' ' }
                    if (str1.length > 2 && !str1.startsWith("!")) {
                        strlistreturn[strlistreturn.size] = str1
                    }
                }
            })
            return strlistreturn
        }

        fun getMultipleCoreDocumentsWaySuggestion(str: Collection<String?>, localNLP: StanfordCoreNLP): ConcurrentMap<String, CoreDocument> {
            val annCollector: AnnotationCollector<Annotation?> = AnnotationCollector<Annotation?>()
            val annotationreturnMap = MapMaker().concurrencyLevel(6).makeMap<String, CoreDocument>()
            runBlocking {
                val job = launch(Dispatchers.Default) {
                    for (exampleString in str) {
                        localNLP.annotate(Annotation(exampleString), annCollector)
                        AnnotationCollector.i++
                    }
                    yield()
                }
                job.join()
            }
            try {
                Thread.sleep(1500)
            } catch (ex: InterruptedException) {
                Logger.getLogger(Datahandler::class.java.name).log(Level.SEVERE, null, ex)
            }
            runBlocking {
                val job1 = launch(Dispatchers.Default) {
                    for (ann in annCollector.annotationsT) {
                        if (ann != null) {
                            ann.compact()
                            val CD = CoreDocument(ann)
                            annotationreturnMap[CD.text()] = CD
                        }
                    }
                    yield()
                }
                job1.join()
            }
            try {
                Thread.sleep(1500)
            } catch (ex: InterruptedException) {
                Logger.getLogger(Datahandler::class.java.name).log(Level.SEVERE, null, ex)
            }
            return annotationreturnMap
        }
    }

    init {
        stopwatch = Stopwatch.createUnstarted()
        jmweAnnotationCache = MapMaker().concurrencyLevel(4).makeMap<String, Annotation>()
        pipelineAnnotationCache = MapMaker().concurrencyLevel(4).makeMap<String, Annotation>()
        pipelineSentimentAnnotationCache = MapMaker().concurrencyLevel(4).makeMap<String, Annotation>()
        coreDocumentAnnotationCache = MapMaker().concurrencyLevel(5).makeMap<String, CoreDocument>()
    }
}